#include "../Framework2/Direct3D/Direct3DApp.h"
#include "../Framework2/Util/Direct3DModel.h"

#include "../Framework2/Util/CollisionSet.h"

#define RT_SIZE 512

#define LIGHT_RANGE 3000.0f
#define INV_LIGHT_RANGE (1.0f / 3000.0f)

#define STR(x) #x
#define DEFINE_STR(x) "#define " #x " " STR(x) "\n"

#define SAMPLE_COUNT 400
#define STATIC_LIGHT_COUNT 3

struct Vertex {
	vec3 pos;
	vec2 texCoord;
	vec3 tangent;
	vec3 binormal;
	vec3 normal;
	vec2 texCoord2;
};

struct PolyData {
	vec3 v0, vDir0, vDir1;
	vec2 t0, tDir0, tDir1;
};

struct Light {
	vec3 pos;
	float size;
	float radius;
	vec3 color;
};

class MainApp : public Direct3DApp {
public:
	void initMenu();
	void resetCamera();
	void processMovement(const vec3 &newPosition);

	bool init();
	bool exit();

	bool load();
	bool unload();

	void setupClipPlanes(const mat4 &itmvp, const vec3 &pos, const float radius);

	void drawLight(const Light &light);
	void drawDistance(const Light &light);
	void drawShadow(const Light &light);
	bool drawFrame();
protected:
	Direct3DModel *model;
	
	Light dynamicLight;

	Light staticLights[STATIC_LIGHT_COUNT];
	TextureID lightMaps[STATIC_LIGHT_COUNT];

	CollisionSet coll;

	ShaderID lightingShader, distanceShader, shadowShader;
	ShaderID staticLightingShader, ambientLightingShader;

	TextureID distance, distanceDepth, shadow, shadowDepth;
	TextureID lightSpot;
	TextureID base[3], bump[3];

	bool drawDynamicLight, drawStaticLights, useClipPlanes;
};
